<?php

class AKUTemplate {
  
  static function factory($id){
    global $REX,$I18N;
    $o = new self();
    if(0==$id){
      $o->data = array('revision'=>0);
    }else{
      // Load Data
      $db = rex_sql::factory();
      $db->setTable($REX['TABLE_PREFIX'].'template');
      $db->setWhere("`id`=$id LIMIT 1");
      $db->select("*");
      if(0==$db->getRows()){
        $o = NULL;
      }else{
        $o->data = $db->getRow();
      }
    }
    return $o;
  }
    
  function setValue($name,$value){
    $this->data[$name] = $value;
  }

  function getValue($name){
    if(isset($this->data[$name])){
      return $this->data[$name];
    }else{
      return '';
    }
  }


  public function getView($name,$viewparams='') {
    global $REX;
    $params = &$viewparams; //backwards compatibility
    $view = $REX['INCLUDE_PATH'].'/addons/aku_admin/src/view.AKUTemplate.'.$name.'.php';
    if(file_exists($view)){
      ob_start();
      include $view;
      return ob_get_clean();
    }else{
      return '';
    }
  }

  public function save(){
    global $REX;
    $db = rex_sql::factory();
    $db->setTable($REX['TABLE_PREFIX'].'template');
    $db->setValue('name',$db->escape($this->getValue('name')));
    $db->setValue('tmpl_type',$db->escape($this->getValue('tmpl_type')));
    $db->setValue('content',$db->escape($this->getValue('content')));
    $db->setValue('active',$this->getValue('active'));
    $db->setValue('attributes',$this->getValue('attributes'));
    if(isset($this->data['id']) && 0 != $this->getValue('id')){
      // update
      $db->setValue('updatedate',time());
      $db->setValue('updateuser',$REX['USER']->getValue('user_id'));
      $db->setWhere("`id`=".$this->getValue('id'));
      $db->update();
      // clear cache
      rex_generateAll();
    }else{
      // insert new
      $db->setValue('createdate',time());
      $db->setValue('createuser',$REX['USER']->getValue('user_id'));
      // set updatedate anyway, needed for sync with file system
      $db->setValue('updatedate',time());
      $db->setValue('updateuser',$REX['USER']->getValue('user_id'));
      $db->insert();
      $this->data['id'] = $db->getLastId();
    }
  }

  public function ShowEdit(){
    global $REX,$I18N;
    if(!$REX['USER']->isAdmin()){
      return array(true,$I18N->msg('no_access'));
    }
    return array(true,$this->getView('edit'));
  }

  public function Delete(){
    global $REX,$I18N;
    if(!$REX['USER']->isAdmin()){
      return array(false,$I18N->msg('no_access'));
    }
    if($this->getValue('id') == $REX['DEFAULT_TEMPLATE_ID']){
      return array(false,$I18N->msg('template_error_delete_is_default'));
    }
    // check if is in use
    $db = rex_sql::factory();
    $db->setTable($REX['TABLE_PREFIX'].'article');
    $db->setWhere('`template_id`='.$this->getValue('id'));
    $db->select('`id`');
    if(0==$db->getRows()){
      $db->setTable($REX['TABLE_PREFIX'].'template');
      $db->setWhere("`id`=".$this->getValue('id'));
      $db->delete();
      if(''<$this->getValue('tmpl_type') && NULL !== $this->getValue('tmpl_type')){
        // remove template files if available
        aku_admin_rmdir($REX['HTDOCS_PATH'].'/templates/'.$this->getValue('tmpl_type'));
      }
      return array(true,'');
    }else{
      $aId = array();
      while($row = $db->getRow()){
        $aId[] = $row['id'];
      }
      $aId = array_unique($aId);
      return array(false,$I18N->msg('template_error_delete_is_used').': '.implode(',',$aId));
    }
  }

  public function Activate(){
    global $REX,$I18N;
    if(!$REX['USER']->isAdmin()){
      return array(false,$I18N->msg('no_access'));
    }
    $db = rex_sql::factory();
    $db->setTable($REX['TABLE_PREFIX'].'template');
    $db->setValue('active',1);
    $db->setWhere('id='.$this->getValue('id'));
    $db->update();
    return array(true,'');
  }
  public function Deactivate(){
    global $REX,$I18N;
    if(!$REX['USER']->isAdmin()){
      return array(false,$I18N->msg('no_access'));
    }
    $db = rex_sql::factory();
    $db->setTable($REX['TABLE_PREFIX'].'template');
    $db->setValue('active',0);
    $db->setWhere('id='.$this->getValue('id'));
    $db->update();
    return array(true,'');
  }
  public function Replicate(){
    global $REX,$I18N;
    if(!$REX['USER']->isAdmin()){
      return array(false,$I18N->msg('no_access'));
    }
    // copy database content to file system
    if(''!=$this->getValue('tmpl_type')) return array(true,'');
    $tmpl_type = 'template_'.$this->getValue('id');
    $dir = aku_admin_getTemplateDir().$tmpl_type.'/';
    if(!is_dir($dir)){
      mkdir($dir,0755,true);
    }
    file_put_contents($dir.'index.html','');
    file_put_contents($dir.'content.php',$this->getValue('content'));
    $package = 'name : '.$this->getValue('name')."\n";
    $package.= 'tmpl_type : '.$this->getValue('tmpl_type')."\n";
    $package.= 'revision : '.$this->getValue('revision')."\n";
    file_put_contents($dir.'package.yml',$package);
    $this->setValue('tmpl_type',$tmpl_type);
    $content = '<?php include $REX[\'HTDOCS_PATH\'].\'templates/'.$tmpl_type.'/'.
      'content.php\';';
    $this->setValue('content',$content);
    $this->save();
    return array(true,'');
  }

  public function ShowSetup($params=''){
    global $REX;
    $file = $REX['HTDOCS_PATH'].'templates/'.$this->getValue('tmpl_type').'/setup.php';
    if(!file_exists($file)){
      return array(false,'file_not_found');
    }else{
      ob_start();
      include $file;
      return array(true,ob_get_clean());
    }
  }
}





